#include "Main.h"
#include "Font.h"
#include "Game.h"
#include "Input.h"
#include "Map.h"
#include "Oiram.h"



// ----- Global varaibles -----

Game gGame;
u32 gGameSwitch;



// ----- Global functions -----

void GameSetMode (Game *game, s32 mode, s32 data)
{
	switch (game->mode)
	{
	case GAME_MODE_NONE:
		break;

	case GAME_MODE_TITLE:
		ImgFree (&game->imgTitle);
		memClear (gScreen.palette, 256*sizeof(RGB));
		memClear (gScreen.pixels, gScreen.pitch*gScreen.height);
		game->lives = 3;
		break;

	case GAME_MODE_MAP:
		MapExit (&gMap);
		break;

	case GAME_MODE_GAMEOVER:
		memClear (gScreen.palette, 256*sizeof(RGB));
		memClear (gScreen.pixels, gScreen.pitch*gScreen.height);
		break;

	default:
		ASSERT_ALWAYS();
	}

	game->mode = mode;

	switch (mode)
	{
	case GAME_MODE_NONE:
		break;

	case GAME_MODE_TITLE:
		ImgCreateFromBmp (&game->imgTitle, "Title.bmp", TRUE);
		memcpy (gScreen.palette, game->imgTitle.palette, 256*sizeof(RGB));
		clear_keybuf ();
		break;

	case GAME_MODE_MAP:
		game->curLevel = data;
		MapInit (game->curLevel, &gMap);
		break;

	case GAME_MODE_GAMEOVER:
		gScreen.palette[0].r = 0;
		gScreen.palette[0].g = 0;
		gScreen.palette[0].b = 0;
		gScreen.palette[1].r = 63;
		gScreen.palette[1].g = 63;
		gScreen.palette[1].b = 63;
		game->timer = GAME_GAMEOVER_TIME;
		break;

	case GAME_MODE_END:
		gScreen.palette[0].r = 0;
		gScreen.palette[0].g = 0;
		gScreen.palette[0].b = 0;
		gScreen.palette[1].r = 63;
		gScreen.palette[1].g = 63;
		gScreen.palette[1].b = 63;
		game->timer = 60;
		break;

	default:
		ASSERT_ALWAYS();
	}
}

void GameUpdate (Game *game)
{
	if (KEYTRIG(KEY_ESC))
		gQuitGame = TRUE;

	switch (game->mode)
	{
	case GAME_MODE_TITLE:
		ImgBlit (&gScreen, &game->imgTitle, 
			0, 0, 0, 0, 
			game->imgTitle.width, game->imgTitle.height, 
			0);

		if (keypressed ())
			GameSetMode (game, GAME_MODE_MAP, LEVEL_ID_FIRST);
		break;

	case GAME_MODE_MAP:
		MapRender (&gMap, &gScreen);

		MapUpdate (&gMap);

		if (gMap.flags & MAP_FLAG_NEXT_LEVEL)
		{
			MapExit (&gMap);
			game->curLevel++;
			ASSERT (game->curLevel < LEVEL_ID_NUM);
			MapInit (game->curLevel, &gMap);
			MapUpdate (&gMap);
		}
		else if (gMap.flags & MAP_FLAG_DIE)
		{
			if (game->lives == 0)
			{
				GameSetMode (game, GAME_MODE_GAMEOVER, 0);
			}
			else
			{
				game->lives--;
				MapExit (&gMap);
				MapInit (game->curLevel, &gMap);
				MapUpdate (&gMap);
			}
		}
		break;

	case GAME_MODE_GAMEOVER:
		FontPrint16 (&gScreen, 
			gScreen.width/2-strlen("GAME OVER")*16/2, 
			gScreen.height/2-16/2, 
			"GAME OVER", 
			FONT_COLORS(1, 0, 0));
		if (--game->timer == 0)
		{
			GameSetMode (game, GAME_MODE_TITLE, 0);
		}
		break;

	case GAME_MODE_END:
		FontPrint16 (&gScreen, 
			gScreen.width/2-strlen("YOU UNLOCKED IGIUL!")*16/2, 
			gScreen.height/2-16/2, 
			"YOU UNLOCKED IGIUL!", 
			FONT_COLORS(1, 0, 0));

		dOiramOverallsNormal.r = 74>>2;
		dOiramOverallsNormal.g = 128>>2;
		dOiramOverallsNormal.b = 60>>2;
		dOiramOverallsFlower.r = 34>>2;
		dOiramOverallsFlower.g = 220>>2;
		dOiramOverallsFlower.b = 34>>2;
		if (--game->timer == 0)
		{
			GameSetMode (game, GAME_MODE_MAP, LEVEL_ID_FIRST);
		}
		break;

	default:
		ASSERT_ALWAYS();
	}
}
