// ----- Change log -----
// 02/26/05 - Created
// ----------------------

#ifndef MAPDATA_H
#define MAPDATA_H

#include "Main.h"



// ----- Sprite types -----

typedef enum _SPRITE_TYPE
{
	SPRITE_TYPE_OIRAM,
	SPRITE_TYPE_TURTLE,
	SPRITE_TYPE_PIRANHA,
	SPRITE_TYPE_TROLL,
	SPRITE_TYPE_BLOCK,
	SPRITE_TYPE_FLOWER,
	SPRITE_TYPE_SINE,

	SPRITE_TYPE_NUM,
	SPRITE_TYPE_MAX = SPRITE_TYPE_NUM - 1,

} SPRITE_TYPE;

typedef enum _LEVEL_ID
{
	LEVEL_ID_FIRST,
	LEVEL_ID_CASTLE,

	LEVEL_ID_NUM,
	LEVEL_ID_MAX = LEVEL_ID_NUM - 1,

} LEVEL_ID;



// ----- Macros -----

#define SPRITE_IS_OIRAM(x)		((x)->type == SPRITE_TYPE_OIRAM)
#define SPRITE_IS_BADDIE(x)		((x)->type == SPRITE_TYPE_TURTLE || (x)->type == SPRITE_TYPE_PIRANHA || (x)->type == SPRITE_TYPE_TROLL)
#define SPRITE_IS_BLOCK(x)		((x)->type == SPRITE_TYPE_BLOCK)
#define SPRITE_IS_FLOWER(x)		((x)->type == SPRITE_TYPE_FLOWER)
#define SPRITE_IS_ATTACK(x)		((x)->type == SPRITE_TYPE_SINE)



// ----- Structures -----

typedef void (*SptInitFunc)(struct _Sprite *spt, struct _Map *map);
typedef void (*SptUpdateFunc)(struct _Sprite *spt, struct _Map *map);
typedef void (*SptRenderFunc)(struct _Sprite *spt, struct _Map *map, Image *screen);
typedef void (*SptColHandler)(struct _Sprite *spt, struct _Sprite *obs, u32 dir, struct _Map *map);

typedef struct _SptFrame
{
	u8				frame, delay;

} SptFrame;

typedef struct _SptAnim
{
	u8				frameCount;
	SptFrame		frame[];

} SptAnim;

typedef struct _SptTypeFile
{
	s32				hotspotX, hotspotY;
	s32				width, height;
	SptInitFunc		init;
	SptUpdateFunc	update;
	SptRenderFunc	render;
	SptColHandler	colHandler;
	s8Rect			collision;
	u8				flags;
	const char		imageBmpFile[32];
	const SptAnim	**anim;

} SptTypeFile;

typedef struct _SptListFile
{
	u8 spriteType, priority;
	s32 x, y;

} SptListFile;

typedef struct _MapFile
{
	const char			mapBmpFile[32];
	const char			mapBgBmpFile[32];
	const char			tileBmpFile[32];
	s32					xScrollMin, yScrollMin;
	s32					xScrollMax, yScrollMax;
	s32					bgXScrollMul, bgYScrollMul;
	s32					bgXScrollOfs, bgYScrollOfs;
	u8					spriteCount;
	u8					followSprite;
	const SptListFile	*spriteList;
	const u8			*tileCollision;

} MapFile;



// ----- Data -----

extern const SptTypeFile dSptType[SPRITE_TYPE_NUM];
extern const MapFile dMap[LEVEL_ID_NUM];



#endif	// MAPDATA_H