#include "Main.h"
#include "Map.h"
#include "MapData.h"
#include "SptSine.h"



// ----- Local function prototypes -----

static void SineSetMode (Sprite *spt, u32 mode, u32 state);



// ----- Global functions -----

void SptInitSine (Sprite *spt, Map *map)
{
	SineSetMode (spt, SINE_MODE_IDLE, SINE_IDLE_STATE_IDLE);
}

void SptUpdateSine (Sprite *spt, Map *map)
{
	switch (spt->vars.sine.mode)
	{
	case SINE_MODE_IDLE:
		switch (spt->vars.sine.state)
		{
		case SINE_IDLE_STATE_IDLE:
			if (spt->flags & SPRITE_FLAG_HFLIP)
				spt->vars.sine.phase -= SINE_PHASE_SPEED;
			else
				spt->vars.sine.phase += SINE_PHASE_SPEED;

			if (--spt->vars.sine.life == 0)
				spt->flags &= ~SPRITE_FLAG_ALIVE;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		ASSERT_ALWAYS();
	}
}

void SptColSine (Sprite *spt, Sprite *obs, u32 dir, Map *map)
{
	if (obs == NULL)	// Map collision
	{
		spt->flags &= ~SPRITE_FLAG_ALIVE;
	}
	else	// Sprite collision
	{
		if (!SPRITE_IS_OIRAM(obs))
		{
			spt->flags &= ~SPRITE_FLAG_ALIVE;
		}
	}
}

void SptRenderSine (Sprite *spt, Map *map, Image *screen)
{
	s32 i, angle;

	angle = spt->vars.sine.phase;

	for (i = 0; i < spt->width*2; i++)
	{
		s32 x, y;
		float sineVal;

		sineVal = sin( (float)angle * 2.0*3.14159265 / (float)SINE_FREQ_FX(16) );

		x = spt->x + SPT_FX(i - spt->width)/2;
		y = spt->y + (s32) ((float)SPT_FX(spt->height)/2 * sineVal);

		x = SPT_FX_TO_INT(x) - map->xScroll;
		y = SPT_FX_TO_INT(y) - map->yScroll;

		if (x >= 0 && x < screen->width && 
			y >= y && y < screen->height)
		{
			screen->pixels[y*screen->pitch + x] = spt->vars.sine.color;
		}

		angle += spt->vars.sine.freq;
	}
}



// ----- Local functions -----

static void SineSetMode (Sprite *spt, u32 mode, u32 state)
{
	ASSERT (mode < SINE_MODE_NUM);

	spt->vars.sine.mode		= mode;
	spt->vars.sine.state	= state;

	switch (mode)
	{
	case SINE_MODE_IDLE:
		switch (state)
		{
		case SINE_IDLE_STATE_IDLE:
			spt->vars.sine.life = SINE_LIFE_TIME;
			break;

		default:
			ASSERT_ALWAYS();
		}
		break;

	default:
		break;
	}
}
